@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: escape.c,v 1.25 2003/01/29 10:14:20 bagder Exp $
+ * $Id: escape.c,v 1.30 2004/03/08 08:38:29 bagder Exp $
  ***************************************************************************/
 
 /* Escape and unescape URL encoding in strings. The functions return a new
@@ -33,18 +33,18 @@
 #include <string.h>
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
 char *curl_escape(const char *string, int length)
 {
-  int alloc = (length?length:(int)strlen(string))+1;  
+  size_t alloc = (length?(size_t)length:strlen(string))+1;  
   char *ns = malloc(alloc);
   char *testing_ptr = NULL;
   unsigned char in;
-  int newlen = alloc;
-  int index=0;
+  size_t newlen = alloc;
+  int strindex=0;
 
   length = alloc-1;
   while(length--) {
@@ -65,27 +65,31 @@ char *curl_escape(const char *string, int length)
           ns = testing_ptr;
         }
       }
-      sprintf(&ns[index], "%%%02X", in);
+      sprintf(&ns[strindex], "%%%02X", in);
 
-      index+=3;
+      strindex+=3;
     }
     else {
       /* just copy this */
-      ns[index++]=in;
+      ns[strindex++]=in;
     }
     string++;
   }
-  ns[index]=0; /* terminate it */
+  ns[strindex]=0; /* terminate it */
   return ns;
 }
 
+#define ishex(in) ((in >= 'a' && in <= 'f') || \
+                   (in >= 'A' && in <= 'F') || \
+                   (in >= '0' && in <= '9'))
+
 char *curl_unescape(const char *string, int length)
 {
   int alloc = (length?length:(int)strlen(string))+1;
   char *ns = malloc(alloc);
   unsigned char in;
-  int index=0;
-  unsigned int hex;
+  int strindex=0;
+  long hex;
  
   if( !ns ) {
     return NULL;
@@ -93,22 +97,31 @@ char *curl_unescape(const char *string, int length)
   
   while(--alloc > 0) {
     in = *string;
-    if('%' == in) {
-      /* encoded part */
-      if(sscanf(string+1, "%02X", &hex)) {
-        in = hex;
-        string+=2;
-        alloc-=2;
-      }
+    if(('%' == in) && ishex(string[1]) && ishex(string[2])) {
+      /* this is two hexadecimal digits following a '%' */
+      char hexstr[3];
+      char *ptr;
+      hexstr[0] = string[1];
+      hexstr[1] = string[2];
+      hexstr[2] = 0;
+
+      hex = strtol(hexstr, &ptr, 16);
+
+      in = (unsigned char)hex; /* this long is never bigger than 255 anyway */
+      string+=2;
+      alloc-=2;
     }
     
-    ns[index++] = in;
+    ns[strindex++] = in;
     string++;
   }
-  ns[index]=0; /* terminate it */
+  ns[strindex]=0; /* terminate it */
   return ns;
 }
 
+/* For operating systems/environments that use different malloc/free
+   ssystems for the app and for this library, we provide a free that uses
+   the library's memory system */
 void curl_free(void *p)
 {
   free(p);
