@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: file.c,v 1.29 2003/01/29 10:14:20 bagder Exp $
+ * $Id: file.c,v 1.53 2004/04/06 15:14:10 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -36,7 +36,6 @@
 #include <errno.h>
 
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock.h>
 #include <time.h>
 #include <io.h>
 #include <fcntl.h>
@@ -48,7 +47,6 @@
 #include <netinet/in.h>
 #endif
 #include <sys/time.h>
-#include <sys/resource.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -83,33 +81,60 @@
 #include "progress.h"
 #include "sendf.h"
 #include "escape.h"
+#include "file.h"
+#include "speedcheck.h"
+#include "getinfo.h"
+#include "transfer.h" /* for Curl_readwrite_init() */
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
 /* Emulate a connect-then-transfer protocol. We connect to the file here */
 CURLcode Curl_file_connect(struct connectdata *conn)
 {
-  char *actual_path = curl_unescape(conn->path, 0);
-  struct FILE *file;
+  char *real_path = curl_unescape(conn->path, 0);
+  struct FILEPROTO *file;
   int fd;
 #if defined(WIN32) || defined(__EMX__)
   int i;
+  char *actual_path;
 #endif
 
-  file = (struct FILE *)malloc(sizeof(struct FILE));
+  file = (struct FILEPROTO *)calloc(sizeof(struct FILEPROTO), 1);
   if(!file)
     return CURLE_OUT_OF_MEMORY;
 
-  memset(file, 0, sizeof(struct FILE));
   conn->proto.file = file;
 
 #if defined(WIN32) || defined(__EMX__)
+  /* If the first character is a slash, and there's
+     something that looks like a drive at the beginning of
+     the path, skip the slash.  If we remove the initial
+     slash in all cases, paths without drive letters end up
+     relative to the current directory which isn't how
+     browsers work.
+
+     Some browsers accept | instead of : as the drive letter
+     separator, so we do too.
+
+     On other platforms, we need the slash to indicate an
+     absolute pathname.  On Windows, absolute paths start
+     with a drive letter.
+  */
+  actual_path = real_path;
+  if ((actual_path[0] == '/') &&
+      actual_path[1] &&
+      (actual_path[2] == ':' || actual_path[2] == '|'))
+  {
+    actual_path[2] = ':';
+    actual_path++;
+  }
+
   /* change path separators from '/' to '\\' for Windows and OS/2 */
   for (i=0; actual_path[i] != '\0'; ++i)
     if (actual_path[i] == '/')
@@ -117,9 +142,9 @@ CURLcode Curl_file_connect(struct connectdata *conn)
 
   fd = open(actual_path, O_RDONLY | O_BINARY);	/* no CR/LF translation! */
 #else
-  fd = open(actual_path, O_RDONLY);
+  fd = open(real_path, O_RDONLY);
 #endif
-  free(actual_path);
+  free(real_path);
 
   if(fd == -1) {
     failf(conn->data, "Couldn't open file %s", conn->path);
@@ -130,6 +155,10 @@ CURLcode Curl_file_connect(struct connectdata *conn)
   return CURLE_OK;
 }
 
+#if defined(WIN32) && (SIZEOF_CURL_OFF_T > 4)
+#define lseek(x,y,z) _lseeki64(x, y, z)
+#endif
+
 /* This is the do-phase, separated from the connect-phase above */
 
 CURLcode Curl_file(struct connectdata *conn)
@@ -141,22 +170,61 @@ CURLcode Curl_file(struct connectdata *conn)
   */
   CURLcode res = CURLE_OK;
   struct stat statbuf;
-  double expected_size=-1;
+  curl_off_t expected_size=0;
+  bool fstated=FALSE;
   ssize_t nread;
   struct SessionHandle *data = conn->data;
   char *buf = data->state.buffer;
-  int bytecount = 0;
-  struct timeval start = Curl_tvnow();
-  struct timeval now = start;
+  curl_off_t bytecount = 0;
   int fd;
+  struct timeval now = Curl_tvnow();
+
+  Curl_readwrite_init(conn);
+  Curl_initinfo(data);
+  Curl_pgrsStartNow(data);
 
   /* get the fd from the connection phase */
   fd = conn->proto.file->fd;
 
-/*VMS?? -- This only works reliable for STREAMLF files */
+  /* VMS: This only works reliable for STREAMLF files */
   if( -1 != fstat(fd, &statbuf)) {
     /* we could stat it, then read out the size */
-    expected_size = (double)statbuf.st_size;
+    expected_size = statbuf.st_size;
+    fstated = TRUE;
+  }
+
+  /* If we have selected NOBODY and HEADER, it means that we only want file
+     information. Which for FILE can't be much more than the file size and
+     date. */
+  if(conn->bits.no_body && data->set.include_header && fstated) {
+    CURLcode result;
+    sprintf(buf, "Content-Length: %" FORMAT_OFF_T "\r\n", expected_size);
+    result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
+    if(result)
+      return result;
+
+    sprintf(buf, "Accept-ranges: bytes\r\n");
+    result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
+    if(result)
+      return result;
+
+#ifdef HAVE_STRFTIME
+    if(fstated) {
+      struct tm *tm;
+      time_t clock = (time_t)statbuf.st_mtime;
+#ifdef HAVE_GMTIME_R
+      struct tm buffer;
+      tm = (struct tm *)gmtime_r(&clock, &buffer);
+#else
+      tm = gmtime(&clock);
+#endif
+      /* format: "Tue, 15 Nov 1994 12:45:26 GMT" */
+      strftime(buf, BUFSIZE-1, "Last-Modified: %a, %d %b %Y %H:%M:%S GMT\r\n",
+               tm);
+      result = Curl_client_write(data, CLIENTWRITE_BOTH, buf, 0);
+    }
+#endif
+    return result;
   }
 
   /* Added by Dolbneff A.V & Spiridonoff A.V */
@@ -166,20 +234,21 @@ CURLcode Curl_file(struct connectdata *conn)
     /* Is this error code suitable in such situation? */
     return CURLE_FTP_BAD_DOWNLOAD_RESUME;
 
-  if (expected_size == 0)
+  if (fstated && (expected_size == 0))
     return CURLE_OK;
 
   /* The following is a shortcut implementation of file reading
      this is both more efficient than the former call to download() and
      it avoids problems with select() and recv() on file descriptors
      in Winsock */
-  if(expected_size != -1)
+  if(fstated)
     Curl_pgrsSetDownloadSize(data, expected_size);
 
   if(conn->resume_from)
-    /* Added by Dolbneff A.V & Spiridonoff A.V */
     lseek(fd, conn->resume_from, SEEK_SET);
 
+  Curl_pgrsTime(data, TIMER_STARTTRANSFER);
+
   while (res == CURLE_OK) {
     nread = read(fd, buf, BUFSIZE-1);
 
@@ -199,11 +268,13 @@ CURLcode Curl_file(struct connectdata *conn)
     if(res)
       return res;
 
-    now = Curl_tvnow();
+    Curl_pgrsSetDownloadCounter(data, bytecount);
+
     if(Curl_pgrsUpdate(conn))
       res = CURLE_ABORTED_BY_CALLBACK;
+    else
+      res = Curl_speedcheck (data, now);
   }
-  now = Curl_tvnow();
   if(Curl_pgrsUpdate(conn))
     res = CURLE_ABORTED_BY_CALLBACK;
 
