@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: formdata.c,v 1.48 2003/04/15 09:29:39 bagder Exp $
+ * $Id: formdata.c,v 1.58 2004/04/23 11:00:47 bagder Exp $
  ***************************************************************************/
 
 /*
@@ -118,13 +118,16 @@ Content-Disposition: form-data; name="FILECONTENT"
 
 #include <time.h>
 
+#ifndef CURL_OLDSTYLE 
+#define CURL_OLDSTYLE 1 /* enable deprecated prototype for curl_formparse */
+#endif
 #include <curl/curl.h>
 #include "formdata.h"
 
 #include "strequal.h"
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
@@ -133,7 +136,7 @@ Content-Disposition: form-data; name="FILECONTENT"
 
 /* What kind of Content-Type to use on un-specified files with unrecognized
    extensions. */
-#define HTTPPOST_CONTENTTYPE_DEFAULT "text/plain"
+#define HTTPPOST_CONTENTTYPE_DEFAULT "application/octet-stream"
 
 /* This is a silly duplicate of the function in main.c to enable this source
    to compile stand-alone for better debugging */
@@ -173,7 +176,7 @@ int FormParse(char *input,
 	      struct curl_httppost **httppost,
 	      struct curl_httppost **last_post)
 {
-  /* nextarg MUST be a string in the format 'name=contents' and we'll
+  /* 'input' MUST be a string in the format 'name=contents' and we'll
      build a linked list with the info */
   char name[256];
   char *contents;
@@ -267,7 +270,7 @@ int FormParse(char *input,
 	    {".jpg",  "image/jpeg"},
 	    {".jpeg", "image/jpeg"},
 	    {".txt",  "text/plain"},
-	    {".html", "text/plain"}
+	    {".html", "text/html"}
 	  };
 
 	  if(prevtype)
@@ -395,12 +398,9 @@ int curl_formparse(char *input,
  *
  ***************************************************************************/
 static struct curl_httppost *
-AddHttpPost(char * name, long namelength,
-            char * value, long contentslength,
-
-            /* CMC: Added support for buffer uploads */
-            char * buffer, long bufferlength,
-
+AddHttpPost(char * name, size_t namelength,
+            char * value, size_t contentslength,
+            char * buffer, size_t bufferlength,
             char *contenttype,
             long flags,
             struct curl_slist* contentHeader,
@@ -414,14 +414,11 @@ AddHttpPost(char * name, long namelength,
   if(post) {
     memset(post, 0, sizeof(struct curl_httppost));
     post->name = name;
-    post->namelength = name?(namelength?namelength:(long)strlen(name)):0;
+    post->namelength = (long)(name?(namelength?namelength:strlen(name)):0);
     post->contents = value;
-    post->contentslength = contentslength;
-
-    /* CMC: Added support for buffer uploads */
+    post->contentslength = (long)contentslength;
     post->buffer = buffer;
-    post->bufferlength = bufferlength;
-
+    post->bufferlength = (long)bufferlength;
     post->contenttype = contenttype;
     post->contentheader = contentHeader;
     post->showfilename = showfilename;
@@ -553,15 +550,16 @@ static const char * ContentTypeForFilename (const char *filename,
  * Returns 0 on success and 1 if the malloc failed.
  *
  ***************************************************************************/
-static int AllocAndCopy (char **buffer, int buffer_length)
+static int AllocAndCopy(char **buffer, size_t buffer_length)
 {
   const char *src = *buffer;
-  int length, add = 0;
+  size_t length;
+  bool add = FALSE;
   if (buffer_length)
     length = buffer_length;
   else {
     length = strlen(*buffer);
-    add = 1;
+    add = TRUE;
   }
   *buffer = (char*)malloc(length+add);
   if (!*buffer)
@@ -790,7 +788,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
         break;
       }
 
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFER:
       {
         char *filename = array_state?array_value:
@@ -819,7 +816,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
         break;
       }
       
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFERPTR:
         current_form->flags |= HTTPPOST_PTRBUFFER;
       if (current_form->buffer)
@@ -834,7 +830,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
       }
       break;
 
-    /* CMC: Added support for buffer uploads */
     case CURLFORM_BUFFERLENGTH:
       if (current_form->bufferlength)
         return_value = CURL_FORMADD_OPTION_TWICE;
@@ -913,7 +908,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
            ( (form->flags & HTTPPOST_FILENAME) &&
              (form->flags & HTTPPOST_PTRCONTENTS) ) ||
 
-           /* CMC: Added support for buffer uploads */
            ( (!form->buffer) &&
              (form->flags & HTTPPOST_BUFFER) &&
              (form->flags & HTTPPOST_PTRBUFFER) ) ||
@@ -943,8 +937,6 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
         if ( !(form->flags & HTTPPOST_FILENAME) &&
              !(form->flags & HTTPPOST_READFILE) && 
              !(form->flags & HTTPPOST_PTRCONTENTS) &&
-
-             /* CMC: Added support for buffer uploads */
              !(form->flags & HTTPPOST_PTRBUFFER) ) {
 
           /* copy value (without strdup; possibly contains null characters) */
@@ -955,10 +947,7 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
         }
         post = AddHttpPost(form->name, form->namelength,
                            form->value, form->contentslength,
-
-                           /* CMC: Added support for buffer uploads */
                            form->buffer, form->bufferlength,
-
                            form->contenttype, form->flags,
                            form->contentheader, form->showfilename,
                            post, httppost,
@@ -986,9 +975,13 @@ CURLFORMcode FormAdd(struct curl_httppost **httppost,
   return return_value;
 }
 
+/*
+ * curl_formadd() is a public API to add a section to the multipart formpost.
+ */
+
 CURLFORMcode curl_formadd(struct curl_httppost **httppost,
-                 struct curl_httppost **last_post,
-                 ...)
+                          struct curl_httppost **last_post,
+                          ...)
 {
   va_list arg;
   CURLFORMcode result;
@@ -998,9 +991,12 @@ CURLFORMcode curl_formadd(struct curl_httppost **httppost,
   return result;
 }
 
-static int AddFormData(struct FormData **formp,
-                       const void *line,
-                       long length)
+/*
+ * AddFormData() adds a chunk of data to the FormData linked list.
+ */
+static size_t AddFormData(struct FormData **formp,
+                          const void *line,
+                          size_t length)
 {
   struct FormData *newform = (struct FormData *)
     malloc(sizeof(struct FormData));
@@ -1025,9 +1021,12 @@ static int AddFormData(struct FormData **formp,
   return length;
 }
 
+/*
+ * AddFormDataf() adds printf()-style formatted data to the formdata chain.
+ */
 
-static int AddFormDataf(struct FormData **formp,
-                        const char *fmt, ...)
+static size_t AddFormDataf(struct FormData **formp,
+                           const char *fmt, ...)
 {
   char s[4096];
   va_list ap;
@@ -1038,13 +1037,16 @@ static int AddFormDataf(struct FormData **formp,
   return AddFormData(formp, s, 0);
 }
 
-
+/*
+ * Curl_FormBoundary() creates a suitable boundary string and returns an
+ * allocated one.
+ */
 char *Curl_FormBoundary(void)
 {
   char *retstring;
   static int randomizer=0; /* this is just so that two boundaries within
 			      the same form won't be identical */
-  int i;
+  size_t i;
 
   static char table16[]="abcdef0123456789";
 
@@ -1067,7 +1069,10 @@ char *Curl_FormBoundary(void)
   return retstring;
 }
 
-/* Used from http.c, this cleans a built FormData linked list */ 
+/*
+ * Curl_formclean() is used from http.c, this cleans a built FormData linked
+ * list
+ */ 
 void Curl_formclean(struct FormData *form)
 {
   struct FormData *next;
@@ -1080,7 +1085,10 @@ void Curl_formclean(struct FormData *form)
   } while((form=next)); /* continue */
 }
 
-/* external function to free up a whole form post chain */
+/*
+ * curl_formfree() is an external function to free up a whole form post
+ * chain
+ */
 void curl_formfree(struct curl_httppost *form)
 {
   struct curl_httppost *next;
@@ -1109,16 +1117,23 @@ void curl_formfree(struct curl_httppost *form)
   } while((form=next)); /* continue */
 }
 
+/*
+ * Curl_getFormData() converts a linked list of "meta data" into a complete
+ * (possibly huge) multipart formdata. The input list is in 'post', while the
+ * output resulting linked lists gets stored in '*finalform'. *sizep will get
+ * the total size of the whole POST.
+ */
+
 CURLcode Curl_getFormData(struct FormData **finalform,
                           struct curl_httppost *post,
-                          int *sizep)
+                          curl_off_t *sizep)
 {
   struct FormData *form = NULL;
   struct FormData *firstform;
   struct curl_httppost *file;
   CURLcode result = CURLE_OK;
 
-  int size =0;
+  size_t size =0;
   char *boundary;
   char *fileboundary=NULL;
   struct curl_slist* curList;
@@ -1184,8 +1199,6 @@ CURLcode Curl_getFormData(struct FormData **finalform,
                               file->contents));
       }
       else if((post->flags & HTTPPOST_FILENAME) ||
-
-              /* CMC: Added support for buffer uploads */
               (post->flags & HTTPPOST_BUFFER)) {
 
         size += AddFormDataf(&form,
@@ -1229,13 +1242,16 @@ CURLcode Curl_getFormData(struct FormData **finalform,
         /* we should include the contents from the specified file */
         FILE *fileread;
         char buffer[1024];
-        int nread;
+        size_t nread;
+
+        fileread = strequal("-", file->contents)?
+          stdin:fopen(file->contents, "rb"); /* binary read for win32  */
+        /*
+         * VMS: This only allows for stream files on VMS.  Stream files are
+         * OK, as are FIXED & VAR files WITHOUT implied CC For implied CC,
+         * every record needs to have a \n appended & 1 added to SIZE
+         */
 
-        fileread = strequal("-", file->contents)?stdin:
-          /* binary read for win32 crap */
-          /*VMS??*/ fopen(file->contents, "rb");  /* ONLY ALLOWS FOR STREAM FILES ON VMS */
-        /*VMS?? Stream files are OK, as are FIXED & VAR files WITHOUT implied CC */
-        /*VMS?? For implied CC, every record needs to have a \n appended & 1 added to SIZE */
         if(fileread) {
           while((nread = fread(buffer, 1, 1024, fileread)))
             size += AddFormData(&form, buffer, nread);
@@ -1244,18 +1260,14 @@ CURLcode Curl_getFormData(struct FormData **finalform,
             fclose(fileread);
         }
         else {
-#if 0
-          /* File wasn't found, add a nothing field! */
-          size += AddFormData(&form, "", 0);
-#endif
           Curl_formclean(firstform);
           free(boundary);
           *finalform = NULL;
           return CURLE_READ_ERROR;
         }
 
-        /* CMC: Added support for buffer uploads */
-      } else if (post->flags & HTTPPOST_BUFFER) {
+      } 
+      else if (post->flags & HTTPPOST_BUFFER) {
           /* include contents of buffer */
           size += AddFormData(&form, post->buffer, post->bufferlength);
       }
@@ -1291,6 +1303,10 @@ CURLcode Curl_getFormData(struct FormData **finalform,
   return result;
 }
 
+/*
+ * Curl_FormInit() inits the struct 'form' points to with the 'formdata'
+ * and resets the 'sent' counter.
+ */
 int Curl_FormInit(struct Form *form, struct FormData *formdata )
 {
   if(!formdata)
@@ -1302,15 +1318,18 @@ int Curl_FormInit(struct Form *form, struct FormData *formdata )
   return 0;
 }
 
-/* fread() emulation */
-int Curl_FormReader(char *buffer,
-                    size_t size,
-                    size_t nitems,
-                    FILE *mydata)
+/*
+ * Curl_FormReader() is the fread() emulation function that will be used to
+ * deliver the formdata to the transfer loop and then sent away to the peer.
+ */
+size_t Curl_FormReader(char *buffer,
+                       size_t size,
+                       size_t nitems,
+                       FILE *mydata)
 {
   struct Form *form;
-  int wantedsize;
-  int gotsize = 0;
+  size_t wantedsize;
+  size_t gotsize = 0;
 
   form=(struct Form *)mydata;
 
@@ -1348,48 +1367,25 @@ int Curl_FormReader(char *buffer,
   return gotsize;
 }
 
-/* possible (old) fread() emulation that copies at most one line */
-int Curl_FormReadOneLine(char *buffer,
-                         size_t size,
-                         size_t nitems,
-                         FILE *mydata)
+/*
+ * Curl_formpostheader() returns the first line of the formpost, the
+ * request-header part (which is not part of the request-body like the rest of
+ * the post).
+ */
+char *Curl_formpostheader(void *formp, size_t *len)
 {
-  struct Form *form;
-  int wantedsize;
-  int gotsize;
-
-  form=(struct Form *)mydata;
-
-  wantedsize = size * nitems;
+  char *header;
+  struct Form *form=(struct Form *)formp;
 
   if(!form->data)
-    return -1; /* nothing, error, empty */
+    return 0; /* nothing, ERROR! */
 
-  do {
-  
-    if( (form->data->length - form->sent ) > wantedsize ) {
+  header = form->data->line;
+  *len = form->data->length;
 
-      memcpy(buffer, form->data->line + form->sent, wantedsize);
+  form->data = form->data->next; /* advance */
 
-      form->sent += wantedsize;
-
-      return wantedsize;
-    }
-
-    memcpy(buffer,
-           form->data->line + form->sent,
-           gotsize = (form->data->length - form->sent) );
-
-    form->sent = 0;
-
-    form->data = form->data->next; /* advance */
-
-  } while(!gotsize && form->data);
-  /* If we got an empty line and we have more data, we proceed to the next
-     line immediately to avoid returning zero before we've reached the end.
-     This is the bug reported November 22 1999 on curl 6.3. (Daniel) */
-
-  return gotsize;
+  return header;
 }
 
 
@@ -1440,7 +1436,7 @@ int main()
   int value6length = strlen(value5);
   int errors = 0;
   int size;
-  int nread;
+  size_t nread;
   char buffer[4096];
   struct curl_httppost *httppost=NULL;
   struct curl_httppost *last_post=NULL;
