@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * $Id: mprintf.c,v 1.23 2003/01/29 10:14:23 bagder Exp $
+ * $Id: mprintf.c,v 1.39 2004/03/23 15:25:54 bagder Exp $
  *
  *************************************************************************
  *
@@ -38,17 +38,20 @@
 #include <ctype.h>
 #include <string.h>
 
-#ifndef SIZEOF_LONG_LONG
-/* prevents warnings on picky compilers */
-#define SIZEOF_LONG_LONG 0
-#endif
+#include <curl/mprintf.h>
+
 #ifndef SIZEOF_LONG_DOUBLE
 #define SIZEOF_LONG_DOUBLE 0
 #endif
 
+#ifdef DPRINTF_DEBUG
+#define HAVE_LONGLONG
+#define LONG_LONG long long
+#define ENABLE_64BIT
+#endif
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
@@ -119,17 +122,17 @@ enum {
 typedef struct {
   FormatType type;
   int flags;
-  int width;     /* width OR width parameter number */
-  int precision; /* precision OR precision parameter number */
+  long width;     /* width OR width parameter number */
+  long precision; /* precision OR precision parameter number */
   union {
     char *str;
     void *ptr;
     long num;
-#if SIZEOF_LONG_LONG /* if this is non-zero */
-    long long lnum;
+#ifdef ENABLE_64BIT
+    LONG_LONG lnum;
 #endif
     double dnum;
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE */
     long double ldnum;
 #endif
   } data;
@@ -149,7 +152,7 @@ struct asprintf {
 
 int curl_msprintf(char *buffer, const char *format, ...);
 
-static int dprintf_DollarString(char *input, char **end)
+static long dprintf_DollarString(char *input, char **end)
 {
   int number=0;
   while(isdigit((int)*input)) {
@@ -170,7 +173,8 @@ static BOOL dprintf_IsQualifierNoDollar(char c)
   case '-': case '+': case ' ': case '#': case '.':
   case '0': case '1': case '2': case '3': case '4':
   case '5': case '6': case '7': case '8': case '9':
-  case 'h': case 'l': case 'L': case 'Z': case 'q':
+  case 'h': case 'l': case 'L': case 'z': case 'q':
+  case '*': case 'O':
     return TRUE;
   default:
     return FALSE;
@@ -277,16 +281,17 @@ int dprintf_Pass1Report(va_stack_t *vto, int max)
  *
  ******************************************************************/
 
-static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list arglist)
+static long dprintf_Pass1(char *format, va_stack_t *vto, char **endpos,
+                          va_list arglist)
 {
   char *fmt = format;
   int param_num = 0;
-  int this_param;
-  int width;
-  int precision;
+  long this_param;
+  long width;
+  long precision;
   int flags;
-  int max_param=0;
-  int i;
+  long max_param=0;
+  long i;
 
   while (*fmt) {
     if (*fmt++ == '%') {
@@ -373,12 +378,21 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	case 'q':
 	  flags |= FLAGS_LONGLONG;
 	  break;
-	case 'Z':
-	  if (sizeof(size_t) > sizeof(unsigned long int))
+	case 'z':
+          /* the code below generates a warning if -Wunreachable-code is
+             used */
+	  if (sizeof(size_t) > sizeof(unsigned long))
 	    flags |= FLAGS_LONGLONG;
 	  if (sizeof(size_t) > sizeof(unsigned int))
 	    flags |= FLAGS_LONG;
 	  break;
+        case 'O':
+#if SIZEOF_CURL_OFF_T > 4
+          flags |= FLAGS_LONGLONG;
+#else
+          flags |= FLAGS_LONG;
+#endif
+          break;
 	case '0':
 	  if (!(flags & FLAGS_LEFT))
 	    flags |= FLAGS_PAD_NIL;
@@ -448,13 +462,21 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
       case 'f':
 	vto[i].type = FORMAT_DOUBLE;
 	break;
-      case 'e': case 'E':
+      case 'e':
 	vto[i].type = FORMAT_DOUBLE;
-	flags |= FLAGS_FLOATE| (('E' == *fmt)?FLAGS_UPPER:0);
+	flags |= FLAGS_FLOATE;
 	break;
-      case 'g': case 'G':
+      case 'E':
 	vto[i].type = FORMAT_DOUBLE;
-	flags |= FLAGS_FLOATG| (('G' == *fmt)?FLAGS_UPPER:0);
+	flags |= FLAGS_FLOATE|FLAGS_UPPER;
+	break;
+      case 'g':
+	vto[i].type = FORMAT_DOUBLE;
+	flags |= FLAGS_FLOATG;
+	break;	
+      case 'G':
+	vto[i].type = FORMAT_DOUBLE;
+	flags |= FLAGS_FLOATG|FLAGS_UPPER;
 	break;	
       default:
 	vto[i].type = FORMAT_UNKNOWN;
@@ -516,9 +538,9 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	break;
 	
       case FORMAT_INT:
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
 	if(vto[i].flags & FLAGS_LONGLONG)
-	  vto[i].data.lnum = va_arg(arglist, long long);
+	  vto[i].data.lnum = va_arg(arglist, LONG_LONG);
 	else
 #endif
 	  if(vto[i].flags & FLAGS_LONG)
@@ -528,7 +550,7 @@ static int dprintf_Pass1(char *format, va_stack_t *vto, char **endpos, va_list a
 	break;
 	
       case FORMAT_DOUBLE:
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE */
 	if(vto[i].flags & FLAGS_LONG)
 	  vto[i].data.ldnum = va_arg(arglist, long double);
 	else
@@ -567,7 +589,7 @@ static int dprintf_formatf(
   char *f;
 
   /* Number of characters written.  */
-  register size_t done = 0;
+  int done = 0;
 
   long param; /* current parameter to read */
   long param_num=0; /* parameter counter */
@@ -592,7 +614,8 @@ static int dprintf_formatf(
     char alt;
     
     /* Width of a field.  */
-    register long width;
+    long width;
+
     /* Precision of a field.  */
     long prec;
     
@@ -603,8 +626,8 @@ static int dprintf_formatf(
     long base;
 
     /* Integral values to be written.  */
-#if SIZEOF_LONG_LONG
-    unsigned long long num;
+#ifdef ENABLE_64BIT
+    unsigned LONG_LONG num;
 #else
     unsigned long num;
 #endif
@@ -659,7 +682,7 @@ static int dprintf_formatf(
     else
       prec = -1;
 
-    alt = p->flags & FLAGS_ALT;
+    alt = (p->flags & FLAGS_ALT)?TRUE:FALSE;
     
     switch (p->type) {
     case FORMAT_INT:
@@ -696,9 +719,9 @@ static int dprintf_formatf(
       /* Decimal integer.  */
       base = 10;
 
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
       if(p->flags & FLAGS_LONGLONG) {
-	 /* long long */
+        /* long long */
 	is_neg = p->data.lnum < 0;
 	num = is_neg ? (- p->data.lnum) : p->data.lnum;
       }
@@ -720,7 +743,7 @@ static int dprintf_formatf(
       /* Number of base BASE.  */
       {
 	char *workend = &work[sizeof(work) - 1];
-	register char *w;
+	char *w;
 	
 	/* Supply a default precision if none was given.  */
 	if (prec == -1)
@@ -849,7 +872,7 @@ static int dprintf_formatf(
 	else {
 	  /* Write "(nil)" for a nil pointer.  */
 	  static char strnil[] = "(nil)";
-	  register char *point;
+	  char *point;
 	  
 	  width -= sizeof(strnil) - 1;
 	  if (p->flags & FLAGS_LEFT)
@@ -894,11 +917,11 @@ static int dprintf_formatf(
 
 	if(width >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += curl_msprintf(fptr, "%d", width);
+	  fptr += curl_msprintf(fptr, "%ld", width);
 	}
 	if(prec >= 0) {
 	  /* RECURSIVE USAGE */
-	  fptr += curl_msprintf(fptr, ".%d", prec);
+	  fptr += curl_msprintf(fptr, ".%ld", prec);
 	}
 	if (p->flags & FLAGS_LONG)
 	  strcat(fptr, "l");
@@ -912,7 +935,7 @@ static int dprintf_formatf(
 
 	/* NOTE NOTE NOTE!! Not all sprintf() implementations returns number
 	   of output characters */
-#if SIZEOF_LONG_DOUBLE
+#if 0 /*SIZEOF_LONG_DOUBLE*/
 	if (p->flags & FLAGS_LONG)
 	  /* This is for support of the 'long double' type */
 	  (sprintf)(work, formatbuf, p->data.ldnum);
@@ -927,17 +950,17 @@ static int dprintf_formatf(
 
     case FORMAT_INTPTR:
       /* Answer the count of characters written.  */
-#if SIZEOF_LONG_LONG
+#ifdef ENABLE_64BIT
       if (p->flags & FLAGS_LONGLONG)
-	*(long long int *) p->data.ptr = done;
+	*(LONG_LONG *) p->data.ptr = (LONG_LONG)done;
       else
 #endif
 	if (p->flags & FLAGS_LONG)
-	  *(long int *) p->data.ptr = done;
+	  *(long *) p->data.ptr = (long)done;
       else if (!(p->flags & FLAGS_SHORT))
-	*(int *) p->data.ptr = done;
+	*(int *) p->data.ptr = (int)done;
       else
-	*(short int *) p->data.ptr = done;
+	*(short *) p->data.ptr = (short)done;
       break;
 
     default:
@@ -964,41 +987,38 @@ static int addbyter(int output, FILE *data)
   return -1;
 }
 
+int curl_mvsnprintf(char *buffer, size_t maxlength, const char *format,
+                    va_list ap_save)
+{
+  int retcode;
+  struct nsprintf info;
+
+  info.buffer = buffer;
+  info.length = 0;
+  info.max = maxlength;
+
+  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
+  if(info.max) {
+    /* we terminate this with a zero byte */
+    if(info.max == info.length)
+      /* we're at maximum, scrap the last letter */
+      info.buffer[-1] = 0;
+    else
+      info.buffer[0] = 0;
+  }
+  return retcode;
+}
+
 int curl_msnprintf(char *buffer, size_t maxlength, const char *format, ...)
 {
+  int retcode;
   va_list ap_save; /* argument pointer */
-  int retcode;
-  struct nsprintf info;
-
-  info.buffer = buffer;
-  info.length = 0;
-  info.max = maxlength;
-
   va_start(ap_save, format);
-  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
+  retcode = curl_mvsnprintf(buffer, maxlength, format, ap_save);
   va_end(ap_save);
-  info.buffer[0] = 0; /* we terminate this with a zero byte */
-
-  /* we could even return things like */
-  
   return retcode;
 }
 
-int curl_mvsnprintf(char *buffer, size_t maxlength, const char *format, va_list ap_save)
-{
-  int retcode;
-  struct nsprintf info;
-
-  info.buffer = buffer;
-  info.length = 0;
-  info.max = maxlength;
-
-  retcode = dprintf_formatf(&info, addbyter, format, ap_save);
-  info.buffer[0] = 0; /* we terminate this with a zero byte */
-  return retcode;
-}
-
-
 /* fputc() look-alike */
 static int alloc_addbyter(int output, FILE *data)
 {
@@ -1023,7 +1043,7 @@ static int alloc_addbyter(int output, FILE *data)
     infop->alloc *= 2;
   }
 
-  infop->buffer[ infop->len ] = output;
+  infop->buffer[ infop->len ] = (char)output;
 
   infop->len++;
 
@@ -1146,14 +1166,16 @@ int main()
 {
   char buffer[129];
   char *ptr;
-#if SIZEOF_LONG_LONG>0
-  long long hullo;
-  dprintf("%3$12s %1$s %2$qd %4$d\n", "daniel", hullo, "stenberg", 65);
+#ifdef ENABLE_64BIT
+  long long one=99;
+  long long two=100;
+  long long test = 0x1000000000LL;
+  curl_mprintf("%lld %lld %lld\n", one, two, test);
 #endif
 
-  mprintf("%3d %5d\n", 10, 1998);
+  curl_mprintf("%3d %5d\n", 10, 1998);
   
-  ptr=maprintf("test this then baby %s%s%s%s%s%s %d %d %d loser baby get a hit in yer face now!", "", "pretty long string pretty long string pretty long string pretty long string pretty long string", "/", "/", "/", "pretty long string", 1998, 1999, 2001);
+  ptr=curl_maprintf("test this then baby %s%s%s%s%s%s %d %d %d loser baby get a hit in yer face now!", "", "pretty long string pretty long string pretty long string pretty long string pretty long string", "/", "/", "/", "pretty long string", 1998, 1999, 2001);
 
   puts(ptr);
 
@@ -1162,15 +1184,15 @@ int main()
   free(ptr);
 
 #if 1
-  mprintf(buffer, "%s %s %d", "daniel", "stenberg", 19988);
+  curl_mprintf(buffer, "%s %s %d", "daniel", "stenberg", 19988);
   puts(buffer);
 
-  mfprintf(stderr, "%s %#08x\n", "dummy", 65);
+  curl_mfprintf(stderr, "%s %#08x\n", "dummy", 65);
 
   printf("%s %#08x\n", "dummy", 65);
   {
     double tryout = 3.14156592;
-    mprintf(buffer, "%.2g %G %f %e %E", tryout, tryout, tryout, tryout, tryout);
+    curl_mprintf(buffer, "%.2g %G %f %e %E", tryout, tryout, tryout, tryout, tryout);
     puts(buffer);
     printf("%.2g %G %f %e %E\n", tryout, tryout, tryout, tryout, tryout);
   }
