@@ -1,29 +1,38 @@
-/*****************************************************************************
+/***************************************************************************
  *                                  _   _ ____  _     
  *  Project                     ___| | | |  _ \| |    
  *                             / __| | | | |_) | |    
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
- *
- * In order to be useful for every potential user, curl and libcurl are
- * dual-licensed under the MPL and the MIT/X-derivate licenses.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
+ * This software is licensed as described in the file COPYING, which
+ * you should have received as part of this distribution. The terms
+ * are also available at http://curl.haxx.se/docs/copyright.html.
+ * 
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
- * furnished to do so, under the terms of the MPL or the MIT/X-derivate
- * licenses. You may pick one of these licenses.
+ * furnished to do so, under the terms of the COPYING file.
  *
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: writeout.c,v 1.10 2002/03/19 07:54:55 bagder Exp $
- *****************************************************************************/
+ * $Id: writeout.c,v 1.21 2004/02/25 15:41:36 bagder Exp $
+ ***************************************************************************/
+
+#include "setup.h"
 
 #include <stdio.h>
 #include <string.h>
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
 #include <curl/curl.h>
 
 #define _MPRINTF_REPLACE /* we want curl-functions instead of native ones */
@@ -71,7 +80,7 @@ static struct variable replacements[]={
   {"speed_download", VAR_SPEED_DOWNLOAD},
   {"speed_upload", VAR_SPEED_UPLOAD},
   {"content_type", VAR_CONTENT_TYPE},
-  {NULL, 0}
+  {NULL, VAR_NONE}
 };
 
 void ourWriteOut(CURL *curl, char *writeinfo)
@@ -99,27 +108,28 @@ void ourWriteOut(CURL *curl, char *writeinfo)
           keepit=*end;
           *end=0; /* zero terminate */
           for(i=0; replacements[i].name; i++) {
-            if(strequal(ptr, replacements[i].name)) {
+            if(curl_strequal(ptr, replacements[i].name)) {
               switch(replacements[i].id) {
               case VAR_EFFECTIVE_URL:
-                if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &stringp))
+                if((CURLE_OK ==
+                    curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &stringp))
+                   && stringp)
                   fputs(stringp, stream);
                 break;
               case VAR_HTTP_CODE:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_HTTP_CODE, &longinfo))
-                  fprintf(stream, "%03d", longinfo);
+                   curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &longinfo))
+                  fprintf(stream, "%03ld", longinfo);
                 break;
               case VAR_HEADER_SIZE:
                 if(CURLE_OK ==
                    curl_easy_getinfo(curl, CURLINFO_HEADER_SIZE, &longinfo))
-                  fprintf(stream, "%d", longinfo);
+                  fprintf(stream, "%ld", longinfo);
                 break;
               case VAR_REQUEST_SIZE:
                 if(CURLE_OK ==
                    curl_easy_getinfo(curl, CURLINFO_REQUEST_SIZE, &longinfo))
-                  fprintf(stream, "%d", longinfo);
+                  fprintf(stream, "%ld", longinfo);
                 break;
               case VAR_TOTAL_TIME:
                 if(CURLE_OK ==
@@ -168,11 +178,13 @@ void ourWriteOut(CURL *curl, char *writeinfo)
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
               case VAR_CONTENT_TYPE:
-                if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_CONTENT_TYPE, &stringp))
-                   fputs(stringp, stream);
-               break;
+                if((CURLE_OK ==
+                    curl_easy_getinfo(curl, CURLINFO_CONTENT_TYPE, &stringp))
+                   && stringp)
+                  fputs(stringp, stream);
+                break;
               default:
+                /* -Wunreachable-code wrongly complains on this */
                 break;
               }
               break;
