@@ -1,6 +1,27 @@
 #!/usr/bin/perl
+#***************************************************************************
+#                                  _   _ ____  _
+#  Project                     ___| | | |  _ \| |
+#                             / __| | | | |_) | |
+#                            | (__| |_| |  _ <| |___
+#                             \___|\___/|_| \_\_____|
 #
-# $Id: ftpserver.pl,v 1.35 2003/04/09 12:02:06 bagder Exp $
+# Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://curl.haxx.se/docs/copyright.html.
+#
+# You may opt to use, copy, modify, merge, publish, distribute and/or sell
+# copies of the Software, and permit persons to whom the Software is
+# furnished to do so, under the terms of the COPYING file.
+#
+# This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+# KIND, either express or implied.
+#
+# $Id: ftpserver.pl,v 1.42 2004/04/19 15:20:30 bagder Exp $
+###########################################################################
+
 # This is the FTP server designed for the curl test suite.
 #
 # It is meant to exercise curl, it is not meant to be a fully working
@@ -11,17 +32,22 @@
 #
 
 use Socket;
-use Carp;
 use FileHandle;
 
 use strict;
 
 require "getpart.pm";
 
-open(FTPLOG, ">log/ftpd.log") ||
-    print STDERR "failed to open log file, runs without logging\n";
-
-sub logmsg { print FTPLOG "$$: "; print FTPLOG @_; }
+# open and close each time to allow removal at any time
+sub logmsg {
+    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
+        localtime(time);
+    open(FTPLOG, ">>log/ftpd.log");
+    printf FTPLOG ("%02d:%02d:%02d (%d) ",
+                   $hour, $min, $sec, $$);
+    print FTPLOG @_;
+    close(FTPLOG);
+}
 
 sub ftpmsg {
   # append to the server.input file
@@ -40,12 +66,17 @@ sub ftpmsg {
 my $verbose=0; # set to 1 for debugging
 my $retrweirdo=0;
 my $retrnosize=0;
+my $srcdir=".";
 
 my $port = 8921; # just a default
 do {
     if($ARGV[0] eq "-v") {
         $verbose=1;
     }
+    elsif($ARGV[0] eq "-s") {
+        $srcdir=$ARGV[1];
+        shift @ARGV;
+    }
     elsif($ARGV[0] =~ /^(\d+)$/) {
         $port = $1;
     }
@@ -92,6 +123,7 @@ my %commandok = (
                  'SYST' => 'loggedin',
                  'SIZE' => 'loggedin|twosock',
                  'PWD'  => 'loggedin|twosock',
+                 'MKD'  => 'loggedin|twosock',
                  'QUIT'  => 'loggedin|twosock',
                  'RNFR'  => 'loggedin|twosock',
                  'RNTO'  => 'loggedin|twosock',
@@ -118,6 +150,7 @@ my %displaytext = ('USER' => '331 We are happy you popped in!',
                    'SYST' => '215 UNIX Type: L8', # just fake something
                    'QUIT' => '221 bye bye baby', # just reply something
                    'PWD'  => '257 "/nowhere/anywhere" is current directory',
+                   'MKD'  => '257 Created your requested directory',
                    'REST' => '350 Yeah yeah we set it there for you',
                    'DELE' => '200 OK OK OK whatever you say',
                    'RNFR' => '350 Received your order. Please provide more',
@@ -138,6 +171,12 @@ my %commandfunc = ( 'PORT' => \&PORT_command,
                     'MDTM' => \&MDTM_command,
                     );
 
+
+sub close_dataconn {
+    close(SOCK);
+    logmsg "Closed data connection\n";
+}
+
 my $rest=0;
 sub REST_command {
     $rest = $_[0];
@@ -160,12 +199,12 @@ my @ftpdir=("total 20\r\n",
 "drwxrwxrwx   2 98       1            512 Oct 30 14:33 pub\r\n",
 "dr-xr-xr-x   5 0        1            512 Oct  1  1997 usr\r\n");
 
-    logmsg "$$: pass data to child pid\n";
+    logmsg "pass LIST data on data connection\n";
     for(@ftpdir) {
         print SOCK $_;
     }
-    close(SOCK);
-    logmsg "$$: done passing data to child pid\n";
+    close_dataconn();
+    logmsg "done passing data\n";
 
     print "226 ASCII transfer complete\r\n";
     return 0;
@@ -173,10 +212,11 @@ my @ftpdir=("total 20\r\n",
 
 sub NLST_command {
     my @ftpdir=("file", "with space", "fake", "..", " ..", "funny", "README");
+    logmsg "pass NLST data on data connection\n";
     for(@ftpdir) {
         print SOCK "$_\r\n";
     }
-    close(SOCK);
+    close_dataconn();
     print "226 ASCII transfer complete\r\n";
     return 0;
 }
@@ -184,7 +224,7 @@ sub NLST_command {
 sub MDTM_command {
     my $testno = $_[0];
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
     logmsg "MDTM $testno\n";
 
@@ -211,9 +251,9 @@ sub MDTM_command {
 sub SIZE_command {
     my $testno = $_[0];
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
-    logmsg "SIZE number $testno\n";
+    logmsg "SIZE file \"$testno\"\n";
 
     my @data = getpart("reply", "size");
 
@@ -250,14 +290,17 @@ sub SIZE_command {
 sub RETR_command {
     my $testno = $_[0];
 
-    logmsg "RETR test number $testno\n";
+    logmsg "RETR file \"$testno\"\n";
 
     if($testno =~ /^verifiedserver$/) {
         # this is the secret command that verifies that this actually is
         # the curl test server
-        print "150 Binary junk (10 bytes).\r\n";
+        my $response = "WE ROOLZ: $$\r\n";
+        my $len = length($response);
+        print "150 Binary junk ($len bytes).\r\n";
+        logmsg "pass our pid on the data connection\n";
         print SOCK "WE ROOLZ: $$\r\n";
-        close(SOCK);
+        close_dataconn();
         print "226 File transfer complete\r\n";
         if($verbose) {
             print STDERR "FTPD: We returned proof we are the test server\n";
@@ -266,7 +309,7 @@ sub RETR_command {
         return 0;
     }
 
-    loadtest("data/test$testno");
+    loadtest("$srcdir/data/test$testno");
 
     my @data = getpart("reply", "data");
 
@@ -288,11 +331,12 @@ sub RETR_command {
             "226 File transfer complete\r\n";
             logmsg "150+226 in one shot!\n";
 
+            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
                 print SOCK $send;
             }
-            close(SOCK);
+            close_dataconn();
             $retrweirdo=0; # switch off the weirdo again!
         }
         else {
@@ -304,12 +348,12 @@ sub RETR_command {
             print "150 Binary data connection for $testno () $sz.\r\n";
             logmsg "150 Binary data connection for $testno () $sz.\n";
 
+            logmsg "pass RETR data on data connection\n";
             for(@data) {
                 my $send = $_;
                 print SOCK $send;
             }
-            close(SOCK);
-
+            close_dataconn();
             print "226 File transfer complete\r\n";
         }
     }
@@ -329,6 +373,8 @@ sub STOR_command {
 
     print "125 Gimme gimme gimme!\r\n";
 
+    logmsg "retrieve STOR data on data connection\n";
+
     open(FILE, ">$filename") ||
         return 0; # failed to open output
 
@@ -339,7 +385,7 @@ sub STOR_command {
         print FILE $line;
     }
     close(FILE);
-    close(SOCK);
+    close_dataconn();
 
     logmsg "received $ulsize bytes upload\n";
 
@@ -393,7 +439,7 @@ sub PASV_command {
 
     close(Server2); # close the listener when its served its purpose!
 
-    logmsg "$$: data connection from $name [", inet_ntoa($iaddr), "] at port $iport\n";
+    logmsg "data connection from $name [", inet_ntoa($iaddr), "] at port $iport\n";
 
     return;
 }
@@ -428,6 +474,7 @@ sub PORT_command {
 $SIG{CHLD} = \&REAPER;
 
 my %customreply;
+my %customcount;
 my %delayreply;
 sub customize {
     undef %customreply;
@@ -440,6 +487,11 @@ sub customize {
         if($_ =~ /REPLY ([A-Z]+) (.*)/) {
             $customreply{$1}=$2;
         }
+        if($_ =~ /COUNT ([A-Z]+) (.*)/) {
+            # we blank the customreply for this command when having
+            # been used this number of times
+            $customcount{$1}=$2;
+        }
         elsif($_ =~ /DELAY ([A-Z]+) (\d*)/) {
             $delayreply{$1}=$2;
         }
@@ -478,7 +530,6 @@ for ( $waitedpid = 0;
     open(STDIN,  "<&Client")   || die "can't dup client to stdin";
     open(STDOUT, ">&Client")   || die "can't dup client to stdout";
     
-    FTPLOG->autoflush(1);
     &customize(); # read test control instructions
 
     print @welcome;
@@ -507,7 +558,7 @@ for ( $waitedpid = 0;
         my $FTPARG=$2;
         my $full=$_;
                  
-        logmsg "GOT: ($1) $_\n";
+        logmsg "Received \"$full\"\n";
 
         if($verbose) {
             print STDERR "IN: $full\n";
@@ -524,6 +575,10 @@ for ( $waitedpid = 0;
             # remain in the same state
         }
         else {
+            
+            if($state != $newstate) {
+                logmsg "switch to state $state\n";
+            }
             $state = $newstate;
         }
 
@@ -540,6 +595,10 @@ for ( $waitedpid = 0;
             $text = $displaytext{$FTPCMD};
         }
         else {
+            if($customcount{$FTPCMD} && (!--$customcount{$FTPCMD})) {
+                # used enough number of times, now blank the customreply
+                $customreply{$FTPCMD}="";
+            }
             logmsg "$FTPCMD made to send '$text'\n";
         }
         if($text) {
@@ -555,10 +614,7 @@ for ( $waitedpid = 0;
                 \&$func($FTPARG, $FTPCMD);
             }
         }
-
-        logmsg "set to state $state\n";
             
     } # while(1)
     close(Client);
-    close(Client2);
 }
